<?php

namespace Drupal\Tests\n8n_chat\Functional\Form;

use Drupal\Tests\BrowserTestBase;

/**
 * Tests the N8nChatSettingsForm using Functional tests.
 *
 * This provides full browser testing including form rendering,
 * submission, and user interactions.
 *
 * @group n8n_chat
 */
class N8nChatSettingsFormFunctionalTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['n8n_chat'];

  /**
   * A user with permission to administer site configuration.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create a user with admin permissions.
    $this->adminUser = $this->drupalCreateUser([
      'administer site configuration',
    ]);
  }

  /**
   * Tests that the settings form is accessible and renders correctly.
   */
  public function testSettingsFormAccess() {
    // Test that anonymous users cannot access the form.
    $this->drupalGet('/admin/config/services/n8n-chat');
    $this->assertSession()->statusCodeEquals(403);

    // Test that admin users can access the form.
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/services/n8n-chat');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('n8n Chat');
  }

  /**
   * Tests form submission with valid data.
   */
  public function testFormSubmissionWithValidData() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/services/n8n-chat');

    // Fill out the form with valid data.
    $edit = [
      'enable_global_widget' => TRUE,
      'webhook_url' => 'https://example.com/webhook',
      'widget_title' => 'Test Chat Widget',
      'widget_position' => 'bottom-left',
      'widget_theme' => 'dark',
      'debug_mode' => TRUE,
      'ai_integration' => FALSE,
      'user_context' => TRUE,
      'session_behavior' => 'persist',
      'history_support' => FALSE,
    ];

    $this->submitForm($edit, 'Save configuration');

    // Verify success message.
    $this->assertSession()->pageTextContains('The configuration options have been saved.');

    // Verify the values were saved by reloading the form.
    $this->drupalGet('/admin/config/services/n8n-chat');
    $this->assertSession()->checkboxChecked('enable_global_widget');
    $this->assertSession()->fieldValueEquals('webhook_url', 'https://example.com/webhook');
    $this->assertSession()->fieldValueEquals('widget_title', 'Test Chat Widget');
    $this->assertSession()->fieldValueEquals('widget_position', 'bottom-left');
    $this->assertSession()->fieldValueEquals('widget_theme', 'dark');
  }

  /**
   * Tests form validation with invalid webhook URL.
   */
  public function testFormValidationWithInvalidWebhookUrl() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/services/n8n-chat');

    // Submit form with invalid webhook URL.
    $edit = [
      'enable_global_widget' => TRUE,
      'webhook_url' => 'invalid-url',
    ];

    $this->submitForm($edit, 'Save configuration');

    // Verify error message is displayed.
    // Note: The URL field type in Drupal may show a different validation message
    // than our custom one, so we check for any URL validation error.
    $this->assertSession()->pageTextContains('URL');

    // Verify form was not saved.
    $config = $this->config('n8n_chat.settings');
    $this->assertNotEquals('invalid-url', $config->get('webhook_url'));
  }

  /**
   * Tests that form elements have correct types and options.
   */
  public function testFormElementsStructure() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/services/n8n-chat');

    // Test that required form elements exist.
    $this->assertSession()->fieldExists('enable_global_widget');
    $this->assertSession()->fieldExists('webhook_url');
    $this->assertSession()->fieldExists('widget_title');
    $this->assertSession()->fieldExists('widget_position');
    $this->assertSession()->fieldExists('widget_theme');

    // Test select options.
    $this->assertSession()->optionExists('widget_position', 'bottom-right');
    $this->assertSession()->optionExists('widget_position', 'bottom-left');
    $this->assertSession()->optionExists('widget_position', 'top-right');
    $this->assertSession()->optionExists('widget_position', 'top-left');

    $this->assertSession()->optionExists('widget_theme', 'light');
    $this->assertSession()->optionExists('widget_theme', 'dark');

    $this->assertSession()->optionExists('session_behavior', 'refresh');
    $this->assertSession()->optionExists('session_behavior', 'persist');
    $this->assertSession()->optionExists('session_behavior', 'load_history');
  }

  /**
   * Tests form with empty webhook URL (should be valid).
   */
  public function testFormWithEmptyWebhookUrl() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/services/n8n-chat');

    // Submit form with empty webhook URL.
    $edit = [
      'enable_global_widget' => FALSE,
      'webhook_url' => '',
      'widget_title' => 'Test Widget',
    ];

    $this->submitForm($edit, 'Save configuration');

    // Verify success message (empty URL should be valid).
    $this->assertSession()->pageTextContains('The configuration options have been saved.');
  }

}
