<?php

namespace Drupal\Tests\n8n_chat\Kernel\Form;

use Drupal\Core\Form\FormState;
use Drupal\KernelTests\KernelTestBase;
use Drupal\n8n_chat\Form\N8nChatSettingsForm;

/**
 * Tests the N8nChatSettingsForm using Kernel tests.
 *
 * @group n8n_chat
 * @coversDefaultClass \Drupal\n8n_chat\Form\N8nChatSettingsForm
 */
class N8nChatSettingsFormTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['system', 'n8n_chat'];

  /**
   * The form under test.
   *
   * @var \Drupal\n8n_chat\Form\N8nChatSettingsForm
   */
  protected $form;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installConfig(['n8n_chat']);
    $this->form = N8nChatSettingsForm::create($this->container);
  }

  /**
   * Tests the form ID.
   *
   * @covers ::getFormId
   */
  public function testGetFormId() {
    $this->assertEquals('n8n_chat_settings', $this->form->getFormId());
  }

  /**
   * Tests the editable config names.
   *
   * @covers ::getEditableConfigNames
   */
  public function testGetEditableConfigNames() {
    $method = new \ReflectionMethod($this->form, 'getEditableConfigNames');
    $method->setAccessible(TRUE);
    $result = $method->invoke($this->form);
    $this->assertEquals(['n8n_chat.settings'], $result);
  }

  /**
   * Tests form building with default values.
   *
   * @covers ::buildForm
   */
  public function testBuildFormStructure() {
    $form_state = new FormState();
    $form = $this->form->buildForm([], $form_state);

    // Test that main fieldsets exist.
    $this->assertArrayHasKey('global_settings', $form);
    $this->assertArrayHasKey('appearance', $form);
    $this->assertArrayHasKey('advanced', $form);
    $this->assertArrayHasKey('integration', $form);

    // Test specific form elements.
    $this->assertArrayHasKey('enable_global_widget', $form['global_settings']);
    $this->assertArrayHasKey('webhook_url', $form['global_settings']);
    $this->assertArrayHasKey('widget_title', $form['appearance']);
    $this->assertArrayHasKey('widget_position', $form['appearance']);

    // Test form element types.
    $this->assertEquals('checkbox', $form['global_settings']['enable_global_widget']['#type']);
    $this->assertEquals('url', $form['global_settings']['webhook_url']['#type']);
    $this->assertEquals('textfield', $form['appearance']['widget_title']['#type']);
    $this->assertEquals('select', $form['appearance']['widget_position']['#type']);
  }

  /**
   * Tests form submission saves configuration.
   *
   * @covers ::submitForm
   */
  public function testSubmitForm() {
    $form_state = new FormState();
    $form_state->setValues([
      'enable_global_widget' => TRUE,
      'webhook_url' => 'https://example.com/webhook',
      'widget_title' => 'Test Chat',
      'widget_position' => 'bottom-left',
      'widget_theme' => 'dark',
      'custom_css' => '.chat { color: red; }',
      'debug_mode' => TRUE,
      'ai_integration' => FALSE,
      'user_context' => TRUE,
      'session_behavior' => 'persist',
      'history_support' => FALSE,
    ]);

    $form = [];
    $this->form->submitForm($form, $form_state);

    // Verify configuration was saved.
    $config = $this->config('n8n_chat.settings');
    $this->assertTrue($config->get('enable_global_widget'));
    $this->assertEquals('https://example.com/webhook', $config->get('webhook_url'));
    $this->assertEquals('Test Chat', $config->get('widget_title'));
    $this->assertEquals('bottom-left', $config->get('widget_position'));
    $this->assertEquals('dark', $config->get('widget_theme'));
  }

  /**
   * Tests form validation for invalid webhook URL.
   *
   * @covers ::validateForm
   */
  public function testValidateFormWithInvalidWebhookUrl() {
    $form = [];
    $form_state = new FormState();
    $form_state->setValue('webhook_url', 'invalid-url');

    $this->form->validateForm($form, $form_state);

    $errors = $form_state->getErrors();
    $this->assertArrayHasKey('webhook_url', $errors);
    $this->assertStringContainsString('Please enter a valid webhook URL', (string) $errors['webhook_url']);
  }

  /**
   * Tests form validation with valid webhook URL.
   *
   * @covers ::validateForm
   */
  public function testValidateFormWithValidWebhookUrl() {
    $form = [];
    $form_state = new FormState();
    $form_state->setValue('webhook_url', 'https://example.com/webhook');

    $this->form->validateForm($form, $form_state);

    $errors = $form_state->getErrors();
    $this->assertArrayNotHasKey('webhook_url', $errors);
  }

  /**
   * Tests form validation with empty webhook URL.
   *
   * @covers ::validateForm
   */
  public function testValidateFormWithEmptyWebhookUrl() {
    $form = [];
    $form_state = new FormState();
    $form_state->setValue('webhook_url', '');

    $this->form->validateForm($form, $form_state);

    $errors = $form_state->getErrors();
    $this->assertArrayNotHasKey('webhook_url', $errors);
  }

}
