<?php

namespace Drupal\Tests\n8n_chat\Kernel;

use Drupal\KernelTests\KernelTestBase;

/**
 * Tests the n8n_chat module hooks and theme functions.
 *
 * @group n8n_chat
 */
class N8nChatKernelTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'block',
    'n8n_chat',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installConfig(['n8n_chat']);
  }

  /**
   * Tests hook_help() implementation.
   */
  public function testHookHelp() {
    $route_match = $this->container->get('current_route_match');

    // Test help page for the module.
    $help = n8n_chat_help('help.page.n8n_chat', $route_match);

    $this->assertStringContainsString('About', $help);
    $this->assertStringContainsString('Configuration', $help);
    $this->assertStringContainsString('n8n chat functionality', $help);
    $this->assertStringContainsString('/admin/config/services/n8n-chat', $help);

    // Test non-matching route.
    $help_other = n8n_chat_help('some.other.route', $route_match);
    $this->assertNull($help_other);
  }

  /**
   * Tests hook_theme() implementation.
   */
  public function testHookTheme() {
    $theme_registry = $this->container->get('theme.registry');
    $theme_info = $theme_registry->get();

    $this->assertArrayHasKey('n8n_chat_block', $theme_info);

    $widget_theme = $theme_info['n8n_chat_block'];
    $this->assertArrayHasKey('variables', $widget_theme);

    $expected_variables = [
      'webhook_url' => NULL,
      'widget_title' => NULL,
      'widget_position' => NULL,
      'widget_theme' => NULL,
      'custom_css' => NULL,
    ];

    $this->assertEquals($expected_variables, $widget_theme['variables']);
    $this->assertEquals('n8n-chat-block', $widget_theme['template']);
  }

  /**
   * Tests hook_page_attachments() with global widget enabled.
   */
  public function testHookPageAttachmentsWithGlobalWidget() {
    // Configure the module to enable global widget.
    $config = $this->config('n8n_chat.settings');
    $config->set('enable_global_widget', TRUE);
    $config->set('webhook_url', 'https://example.com/webhook');
    $config->set('widget_title', 'Test Chat');
    $config->set('widget_position', 'bottom-right');
    $config->set('widget_theme', 'light');
    $config->set('debug_mode', TRUE);
    $config->set('session_behavior', 'refresh');
    $config->set('history_support', FALSE);
    $config->save();

    $attachments = [];
    n8n_chat_page_attachments($attachments);

    // Test that library is attached.
    $this->assertArrayHasKey('#attached', $attachments);
    $this->assertContains('n8n_chat/n8n_chat', $attachments['#attached']['library']);

    // Test drupalSettings.
    $this->assertArrayHasKey('drupalSettings', $attachments['#attached']);
    $this->assertArrayHasKey('n8nChat', $attachments['#attached']['drupalSettings']);

    $settings = $attachments['#attached']['drupalSettings']['n8nChat'];
    $this->assertEquals('https://example.com/webhook', $settings['webhookUrl']);
    $this->assertEquals('Test Chat', $settings['widgetTitle']);
    $this->assertEquals('bottom-right', $settings['widgetPosition']);
    $this->assertEquals('light', $settings['widgetTheme']);
    $this->assertEquals(TRUE, $settings['debugMode']);
    $this->assertEquals('refresh', $settings['sessionBehavior']);
    $this->assertEquals(FALSE, $settings['historySupport']);
  }

  /**
   * Tests hook_page_attachments() with global widget disabled.
   */
  public function testHookPageAttachmentsWithGlobalWidgetDisabled() {
    // Configure the module to disable global widget.
    $config = $this->config('n8n_chat.settings');
    $config->set('enable_global_widget', FALSE);
    $config->save();

    $attachments = [];
    n8n_chat_page_attachments($attachments);

    // Test that nothing is attached when global widget is disabled.
    $this->assertEmpty($attachments);
  }

  /**
   * Tests hook_page_attachments() with default values.
   */
  public function testHookPageAttachmentsWithDefaults() {
    // Configure the module with minimal settings.
    $config = $this->config('n8n_chat.settings');
    $config->set('enable_global_widget', TRUE);
    $config->set('webhook_url', 'https://example.com/webhook');
    $config->save();

    $attachments = [];
    n8n_chat_page_attachments($attachments);

    $settings = $attachments['#attached']['drupalSettings']['n8nChat'];

    // Test default values are applied.
    $this->assertEquals('Chat with us', $settings['widgetTitle']);
    $this->assertEquals('bottom-right', $settings['widgetPosition']);
    $this->assertEquals('light', $settings['widgetTheme']);
    $this->assertEquals(FALSE, $settings['debugMode']);
    $this->assertEquals('refresh', $settings['sessionBehavior']);
    $this->assertEquals(FALSE, $settings['historySupport']);
  }

  /**
   * Tests the module configuration schema.
   */
  public function testConfigurationSchema() {
    $config = $this->config('n8n_chat.settings');

    // Test that configuration can be set and retrieved.
    $config->set('enable_global_widget', TRUE);
    $config->set('webhook_url', 'https://example.com/webhook');
    $config->set('widget_title', 'Test Widget');
    $config->set('widget_position', 'bottom-left');
    $config->set('widget_theme', 'dark');
    $config->set('custom_css', '.test { color: red; }');
    $config->set('debug_mode', TRUE);
    $config->set('ai_integration', TRUE);
    $config->set('user_context', TRUE);
    $config->set('session_behavior', 'persist');
    $config->set('history_support', TRUE);
    $config->save();

    // Verify all values are saved correctly.
    $this->assertTrue($config->get('enable_global_widget'));
    $this->assertEquals('https://example.com/webhook', $config->get('webhook_url'));
    $this->assertEquals('Test Widget', $config->get('widget_title'));
    $this->assertEquals('bottom-left', $config->get('widget_position'));
    $this->assertEquals('dark', $config->get('widget_theme'));
    $this->assertEquals('.test { color: red; }', $config->get('custom_css'));
    $this->assertTrue($config->get('debug_mode'));
    $this->assertTrue($config->get('ai_integration'));
    $this->assertTrue($config->get('user_context'));
    $this->assertEquals('persist', $config->get('session_behavior'));
    $this->assertTrue($config->get('history_support'));
  }

  /**
   * Tests the default configuration installation.
   */
  public function testDefaultConfigurationInstallation() {
    $config = $this->config('n8n_chat.settings');

    // Test default values from config/install/n8n_chat.settings.yml.
    $this->assertFalse($config->get('enable_global_widget'));
    $this->assertEquals('', $config->get('webhook_url'));
    $this->assertEquals('Chat with us', $config->get('widget_title'));
    $this->assertEquals('bottom-right', $config->get('widget_position'));
    $this->assertEquals('light', $config->get('widget_theme'));
    $this->assertEquals('', $config->get('custom_css'));
    $this->assertFalse($config->get('debug_mode'));
    $this->assertFalse($config->get('ai_integration'));
    $this->assertFalse($config->get('user_context'));
    $this->assertEquals('refresh', $config->get('session_behavior'));
    $this->assertFalse($config->get('history_support'));
  }

}
